/**
 * @file  errorclass.h 
 * @brief  einfache Fehlerobjekte fr try-throw-catch  
 *
 * Copyright: 1998 <A href="http://www.infnet.verein.de/">Informatik- und Netzwerkverein Ravensburg e.V</A> 
 *              nach GNU-Copyleft                                
 * @author <A href="http://www.infnet.verein.de/">Rudolf Weber</A>
 *
 * Bemerkung..: Stammt aus radiusd                                
 */
#ifndef ERRORCLASS_H
#define ERRORCLASS_H
#include<stdarg.h>
#include<stdio.h>
#include<string.h>

// Fehlerbehandlungsklassen
/**
 * @brief Allgemeine FehlerKlasse
 */
class Error
{
protected:
  char errmsg[128];
public:
  Error() {}
  /**
   * Construktor mit variabel vielen Argumenten wie printf(3)
   */
  Error(const char *msg,...)
      {
	  va_list args;
	  va_start(args,msg);
	  vsnprintf(errmsg,128,msg,args);
	  va_end(args);
      }
  /**
   * liefert Fehlermeldung
   */
  const char *what() { return errmsg; }
};

/**
 * @brief Systemcallerror
 */
class Systemerror: public Error
{
    int errnr;
    const char *icall;
public:
    Systemerror() {}
    /**
     * Konstruktor mit variablel vielen Argumenten
     * unterst&uuml;zt zustzlich noch %m f&uuml;r Systemfehlermeldung
     * @param err Fehlercode errno
     * @param pcall Name des systemcalls
     * @param msg   Fehlermeldungsformatstring
     */
    Systemerror(int err,char *pcall,char *msg,...)
	{
	    va_list args;
	    va_start(args,msg);
	    init(err,pcall,msg,args);
	    va_end(args);
	}

    
    /**
     * Initialisierung mit variablel vielen Argumenten
     * unterst&uuml;zt zustzlich noch %m f&uuml;r Systemfehlermeldung
     * @param err Fehlercode errno
     * @param pcall Name des systemcalls
     * @param msg   Fehlermeldungsformatstring
     */
    void init(int err,char *pcall,char *msg,va_list args)
	{
	    icall = pcall;
	    char format_sp[128];
	    char *format=msg;
	    char *pm=strstr(msg,"%m");
	    register int l = pm-format;
	    if(pm!=NULL && l<120)
	    {
		char *pq,*pz;
		strncpy(format_sp,msg,l);
		pq = strerror(err);
		pz=format_sp+l;
		while(pz-format_sp < 128 && *pq != '\0') *pz++=*pq++;
		pq=pm+2;
		while(pz-format_sp < 128 && *pq != '\0') *pz++=*pq++;
		*pz='\0';
		format = format_sp;
	    }
	    vsnprintf(errmsg,128,format,args);
	}
    /**
     * liefert Errno
     */
    int errornr() { return errnr; }
    /**
     * liefert Name des Systemcalls
     */
    const char *call() { return icall; }
};

/**
 * \brief Syntexerror
 */

class Syntaxerror: public Error
{
public:
  Syntaxerror(const char *m,...):Error()
    {
      va_list args;
      va_start(args,m);
      vsnprintf(errmsg,128,m,args);
      va_end(args);
    };
};

/**
 * \brief Datenbankfehler
 */
class Databaseerror: public Error
{
  long icode;
public:
  Databaseerror(const char *m):Error(m){};
  Databaseerror(const char *dbname,long code,const char *m,...):Error() 
    {
      char format[128];
      va_list args;
      va_start(args,m);
      snprintf(format,128,"%s-CODE %ld: %s",dbname,code,m);
      vsnprintf(errmsg,128,format,args);
      va_end(args);
      icode=code;
    }
  long code() const { return icode; }
};
#endif
