/**
 * @file mseg.h
 * @brief definiert Speichersegmente zur verwendung mit relativen Zeigern
 * @author <A href="http://www.infnet.verein.de/~rw/">Rudolf Weber</A> 2000 
 * nach einer Idee 1995
 */
#ifndef MSEQ_H
#define MSEQ_H
#include<sys/types.h>
#ifndef NULL
#define NULL (void *)0;
#endif

namespace RAMS
{
/**
 * \brief grundlegenes Speichersegment
 *
 * Das Speichersegment verwaltet nur ganz wenige Daten, Anfang, Ende und
 * Laenge<br>
 * Damit kann es als Grundlage fuer relative Zeiger dienen.<br>
 * Bemerkung: Diese Klasse ist Technologieneutral,
 * So kann ein beliebiger Puffer verwaltet werden, z.B:
 * <UL>
 *  <LI>mit read eingelesener Speicher
 *  <LI>mit recv empfangener Speicher
 *  <LI>mit mmap gemappter Speicher
 *  <LI>ein statisch oder dynamisch allokierter Speicher
 *  <LI>...
 * </UL>
 */
class Mseg
{
 protected:
   void *basis; /// anfang
   void *ende;  /// wie der name schon sagt
   size_t len;  /// Laenge
   /// setzen eines segmentes fuer iteratoren
   void setto(char *anfang,size_t lang)
     { basis=anfang;ende=anfang+lang; len=lang;
     }
 public:
   /**
    * initialiiserung auf ein Nullsegemt
    */
   Mseg() { basis=NULL;ende=NULL;len=0;}
   /**
    * initialisierung auf einen Zeiger
    */
    Mseg(char *anfang,size_t lang) { basis=anfang;ende=anfang+lang; len=lang; }
   /**
    * Anfang des Segments
    */
   const void *begin() { return basis; }
   /**
    * Ende des segments
    */
   const void *end()   { return ende; }
   /**
    * Laenge des Segments
    */
   size_t length()     { return len;  }
};

};
#endif
