/**
 * @file once_alloc.h
 * @brief Speicherpool
 * @author <A href="http://www.infnet.verein.de/~rw/">Rudolf Weber</A> 2000 
 */
#ifndef ONCE_ALLOC_H
#define ONCE_ALLOC_H
#include"rpointer.h"
#include"growingmseg.h"

namespace RAMS
{
/**
 * \brief Allokierungsklasse
 *
 * Damit kann man gut Konfigurationsdaten in ein Speichersegment schreiben
 * Diese ist fr write once read many - Daten gegeignet.
 */
class Oncealloc
{
 protected:
    Growingmseg mseg; /// Arbeitstier
    RcharPointer akt; /// aktuelles Objekt
    RcharPointer ao;  /// Aktuelles Ende
    long align;       /// Alignment
 public:
    /**
     * @param ini Zahl der Blcke am Anfang
     * @param a   Alignment
     */
    Oncealloc(int ini=1,long a=4):akt(0,&mseg),ao(0,&mseg)
	{ mseg.grow(ini); align=a;
	}

    /**
     * Bereitet ein neues Objekt vor
     */
    RcharPointer neu()
	{ ao.setoff(align2(align,ao.offset()));
	  akt=ao;
	  return akt;
	}

    /**
     * @brief liefert das Objekt
     */
    RcharPointer aktuell() { return akt; }
    /**
     * @brief liefert das aktuelle Ende des Speichers
     */
    RcharPointer aktende() { return ao; }

    /**
     * @brief fgt das Zeichen an das aktuelle Objekt an.
     *
     * Dieses Operation kann von Scannern aufgerufen werten, um 
     * Zeichenketten nacheinende zu Allokieren, von denen man nicht weiss, wie 
     * gross sie werden
     */
    void grow1(char c) throw(Growingmseg::Error)
	{
	    if(ao.offset()>=mseg.length()) mseg.grow(1);
	    *(ao.ptr())=c;
    	    ao+=1;
	}

    /**
     * @param l Laenge
     *
     * Garantiert, dass mindestens l Zeichen im Segment noch Platz haben 
     */
    void garant(int l) throw(Growingmseg::Error) 
	{
	    if(ao.offset()+l > mseg.length())
	    {
		long al=align2(getpagesize(),l)/getpagesize();
		mseg.grow(al);
	    }
	}

    /**
     * @param l Laenge
     *
     * reserviert noch l Zeichen am aktuellen Objekt
     */
    RcharPointer reserve(int l) throw(Growingmseg::Error)
	{
	    garant(l);
	    ao+=l;
	    return akt;
	}

    /**
     * @param l Laenge
     *
     * allokiert ein leeres Objekt der Laenge l
     */
    RcharPointer alloc(int l) throw(Growingmseg::Error)
	{
	    neu();
	    return reserve(l);
	}

    /**
     * liefert das aktuelle memorysegment
     */
    Mseg *getmseg() { return &mseg; }

    /**
     * liefert die aktuelle Lnge
     */
    long reallength() const { return ao.offset(); }

    /**
     * @param filename
     *
     * schreibt das Segment in die Datei filename
     */
    void write2file(char *filename) throw(Systemerror);
};
}
#endif
