/**
 * @file rppointer.h
 * @brief defines  relative persistent Pointers 
 * @author <A href="http://www.infnet.verein.de/~rw/">Rudolf Weber</A> 2002 
 * nach einer Idee 1995
 * Idee: ein relativer persitenter Pointer ist im wesentlichen ein Offset.
 *       Zur schoeneren Verwaltung gibt es eine Zugriffsstruktur, deren 
 *       Zuweisung die Segmentbasis abzieht.
 */
#ifndef RPPOINTER_H
#define RPPOINTER_H
#include<assert.h>
#include"mseg.h"
#include<sys/types.h>

namespace RAMS
{

 /**
  * \brief Generischer Persistenter Relativer Zeiger
  *
  * Ein Relativer Zeiger ist immer abhaengig von einem Speichersegment
  */
 class RPPointer_s
 {
 protected:
     /// Zugeh&ouml;riges Speichersegment
    Mseg *seg;
     /// Offset in diesem Speichersegment
    off_t &off;
 public:
    /**
     * Initialisierung
     */
    RPPointer_s(Mseg *s,off_t& o):seg(s),off(o){}
    void init(Mseg *s,off_t& o) { seg=s; off=o; }
    off_t offset() const { return off; }
    
    /**
     * Pr&uuml;fung, ob Speicher in das Segment pa&szlig;t
     */
    int ok() { return off < seg->length(); }
    /**
     * Gr&ouml;ssenberechnung
     */
    size_t operator-(RPPointer_s a)
	{
	    assert(a.seg->begin()==seg->begin());
	    return (off-a.off);
	}

    /// for special purposes
    void setoff(off_t o) { off=o; }
    /**
     * Zuweisung eines relativen Zeiger in einem anderen Typ
     */
    RPPointer_s& cast(const RPPointer_s &anderer)
    {
	seg=anderer.seg;
	off=anderer.off;
	return *this;
    }
    void *operator=(void *p)
    {
      off=(char *)p - (char *)(seg->begin());
      assert(ok());
      return p;
    }
 };
};
#endif



